static char SccsId[] = "@(#)edge.cc	1.1 12/12/96 Voronoi";
// edge.cc
// Dec 1996
// Copyright 1996 SFBellenot All rights reserved

#include "edge.h"
#include <X11/Xlib.h>
#include "pad.h"
#include "voronoi.h"

int Site::idCounter;
int Vertex::idCounter;
int Edge::idCounter;

void Edge::Bisect ( Site* s1, Site *s2 )
{
	Real dx, dy, adx, ady;

	eqPoint[0] = s1;
	eqPoint[1] = s2;
	s1->Ref();
	s2->Ref();

	dx = s2->X() - s1->X();
	dy = s2->Y() - s1->Y();
	adx = dx > 0 ? dx : -dx;
	ady = dy > 0 ? dy : -dy;
	c = s1->X() * dx + s1->Y() * dy
		+ (dx * dx + dy * dy) * 0.5;
	if (adx > ady)
	{
		a = 1.0;
		b = dy / dx;
		c /= dx;
	}
	else
	{
		b = 1.0;
		a = dx / dy;
		c /= dy;
	}

	//out_bisector(newedge);
	//nedges += 1;
}

void Edge::Draw ( Voronoi* V )
{
	Vertex* s1;
	Vertex* s2;
	Real x1, x2, y1, y2;
	Real pxmin = V->PXMin();
	Real pxmax = V->PXMax();
	Real pymin = V->PYMin();
	Real pymax = V->PYMax();

	if (a == 1.0 && b >= 0.0)
	{
		s1 = endPoint[1];
		s2 = endPoint[0];
	}
	else
	{
		s1 = endPoint[0];
		s2 = endPoint[1];
	}

	if (a == 1.0)
	{
		y1 = pymin;
		if (s1 != NULL && s1->Y() > pymin)
			y1 = s1->Y();
		if (y1 > pymax)
			return;
		x1 = c - b * y1;
		y2 = pymax;
		if (s2 != NULL && s2->Y() < pymax)
			y2 = s2->Y();
		if (y2 < pymin)
			return;
		x2 = c - b * y2;
		if ((x1 > pxmax & x2 > pxmax) | (x1 < pxmin & x2 < pxmin))
			return;
		if (x1 > pxmax)
		{
			x1 = pxmax;
			y1 = (c - x1) / b;
		}
		if (x1 < pxmin)
		{
			x1 = pxmin;
			y1 = (c - x1) / b;
		}
		if (x2 > pxmax)
		{
			x2 = pxmax;
			y2 = (c - x2) / b;
		}
		if (x2 < pxmin)
		{
			x2 = pxmin;
			y2 = (c - x2) / b;
		}
	}
	else
	{
		x1 = pxmin;
		if (s1 != NULL && s1->X() > pxmin)
			x1 = s1->X();
		if (x1 > pxmax)
			return;
		y1 = c - a * x1;
		x2 = pxmax;
		if (s2 != NULL && s2->X() < pxmax)
			x2 = s2->X();
		if (x2 < pxmin)
			return;
		y2 = c - a * x2;
		if ((y1 > pymax & y2 > pymax) | (y1 < pymin & y2 < pymin))
			return;
		if (y1 > pymax)
		{
			y1 = pymax;
			x1 = (c - y1) / a;
		}
		if (y1 < pymin)
		{
			y1 = pymin;
			x1 = (c - y1) / a;
		}
		if (y2 > pymax)
		{
			y2 = pymax;
			x2 = (c - y2) / a;
		}
		if (y2 < pymin)
		{
			y2 = pymin;
			x2 = (c - y2) / a;
		}
	}

	V->Line(x1, y1, x2, y2);
}

Real Edge::X ( Real y )
{
	//  ax+by=c
	if ( a == 1.0 )
		return c - b * y;
	return (c - y) / a;
}
