#ifndef _EDGE_H_
#define _EDGE_H_

#pragma ident "@(#)edge.h	1.1 12/12/96 Voronoi"
// Dec 1996
// Copyright 1996 SFBellenot All rights reserved

#include "site.h"
#include "vertex.h"

class Voronoi;
const int LEFT_END = 0;
const int RIGHT_END = 1;

class Edge
{
public:
	Edge ()
	{
		a = b = c = 0.0;
		endPoint[0] = NULL;
		endPoint[1] = NULL;
		eqPoint[0] = NULL;
		eqPoint[1] = NULL;
		id = idCounter++;
		refCount = 1;
		endPointCount = 0;
	}
	void EndPoint ( int LeftRight, Vertex* v )
	{
		endPoint[LeftRight] = v;
		endPointCount++;
	}
	Site * LeftSite () { return eqPoint[LEFT_END]; }
	Site * RightSite () { return eqPoint[RIGHT_END]; }
	void Bisect ( Site * s1, Site * s2 );
	int Id() { return id; }
	void Ref() { refCount++; }
	void Deref()
	{
		#if 0
		refCount--;
		if ( refCount == 0 )
		{
			if ( endPoint[0] )
				endPoint[0]->Deref();
			if ( endPoint[1] )
				endPoint[1]->Deref();
			if ( eqPoint[0] )
				eqPoint[0]->Deref();
			if ( eqPoint[1] )
				eqPoint[1]->Deref();
			delete this;
		}
		#endif
	}
	Real A() { return a; }
	Real B() { return b; }
	Real C() { return c; }
	Real X ( Real y );
	void Draw ( Voronoi* p );

private:
	Real a;
	Real b;
	Real c;
	int endPointCount;
	Vertex* endPoint[2];
	Site* eqPoint[2];

	int id;
	int refCount;

	static int idCounter;
};

#endif
