static char SccsId[] = "@(#)halfedge.cc	1.1 12/12/96 Voronoi";
// halfedge.cc
// Dec 1996
// Copyright 1996 SFBellenot All rights reserved

#include "halfedge.h"

int HalfEdge::idCounter = 0;

/* returns 1 if p is to right of halfedge e */
int HalfEdge::RightOf ( Point& p)
{
	Site* topsite;
	int rightOfSite, above, fast;
	Real dxp, dyp, dxs, t1, t2, t3, yl;

	topsite = e->RightSite();
	rightOfSite = p.X() > topsite->X();
	if (rightOfSite && pm == LEFT_END)
		return (1);
	if (!rightOfSite && pm == RIGHT_END)
		return (0);

	if (e->A() == 1.0)
	{
		dyp = p.Y() - topsite->Y();
		dxp = p.X() - topsite->X();
		fast = 0;
		if ((!rightOfSite & e->B() < 0.0) || (rightOfSite & e->B() >= 0.0))
		{
			above = dyp >= e->B() * dxp;
			fast = above;
		}
		else
		{
			above = p.X() + p.Y() * e->B() > e->C();
			if (e->B() < 0.0)
				above = !above;
			if (!above)
				fast = 1;
		}
		if (!fast)
		{
			dxs = topsite->X() - (e->LeftSite())->X();
			above = e->B() * (dxp * dxp - dyp * dyp) <
				dxs * dyp * (1.0 + 2.0 * dxp / dxs + e->B() * e->B());
			if (e->B() < 0.0)
				above = !above;
		}
	}
	else
	{		/* e->B()==1.0 */
		yl = e->C() - e->A() * p.X();
		t1 = p.Y() - yl;
		t2 = p.X() - topsite->X();
		t3 = yl - topsite->Y();
		above = t1 * t1 > t2 * t2 + t3 * t3;
	}
	return (pm == LEFT_END ? above : !above);
}

Vertex* HalfEdge::Intersect ( HalfEdge* el2 )
{
	Edge* e2;
	Edge* eTmp;
	HalfEdge* el;
	Real d, xint, yint;
	int rightOfSite;
	Vertex* vTmp;

	e2 = el2->e;
	if (e == NULL || e2 == NULL)
		return NULL;
	if (e->RightSite() == e2->RightSite() )
		return NULL;

	d = e->A() * e2->B() - e->B() * e2->A();
	if (-1.0e-10 < d && d < 1.0e-10)
		return NULL;

	xint = (e->C() * e2->B() - e2->C() * e->B()) / d;
	yint = (e2->C() * e->A() - e->C() * e2->A()) / d;

	if ((e->RightSite()->Y() < e2->RightSite()->Y()) ||
	    (e->RightSite()->Y() == e2->RightSite()->Y() &&
	     e->RightSite()->X() < e2->RightSite()->X()))
	{
		el = this;
		eTmp = e;
	}
	else
	{
		el = el2;
		eTmp = e2;
	}
	rightOfSite = xint >= eTmp->RightSite()->X();
	if ((rightOfSite && el->pm == LEFT_END) ||
			(!rightOfSite && el->pm == RIGHT_END))
		return NULL;

	vTmp = new Vertex ( xint, yint );
	return vTmp;
}
