#ifndef _HALFEDGE_H_
#define _HALFEDGE_H_

#pragma ident "@(#)halfedge.h	1.1 12/12/96 Voronoi"
// Dec 1996
// Copyright 1996 SFBellenot All rights reserved

#include "edge.h"
#include <iostream.h>

class HalfEdge
{
public:
	HalfEdge ( Edge* E, int LeftRight )
	{
		e = E;
		if ( e != NULL )
			e->Ref();
		v = NULL;
		left = NULL;
		right = NULL;
		inLR = FALSE;
		pm = LeftRight;
		id = ++idCounter;
		refCount = 1;
		endPointCount = 0;
		yStar = 0.0;
	}
	Site * LeftSite ()
	{
		if ( e != NULL )
			return (pm == LEFT_END)? e->LeftSite() : e->RightSite();
		return NULL;
	}
	Site * RightSite ()
	{
		if ( e != NULL )
			return (pm == LEFT_END)? e->RightSite() : e->LeftSite();
		return NULL;
	}
	// LS stuff
	HalfEdge* LeftHalfEdge() { return left; }
	HalfEdge* RightHalfEdge() { return right; }
	void LRDelete()
	{
		if ( ( left == NULL ) || ( right == NULL ) )
			cerr << "LRDelete error" << endl;
		if ( left != NULL )
			left->right = right;
		if ( right != NULL )
			right->left = left;

		left = right = NULL;
		inLR = FALSE;
	}
	void LRInsertRightOf ( HalfEdge* old ) 
	{
		left = old;
		right = old->right;
		old->right = this;
		right->left = this;
		inLR = TRUE;
	}
	void InsertRightOfComplete ( HalfEdge* old ) 
	{
		left = old;
		right = old->right;
		old->right = this;
	}
	int LRCheck()
	{
		if ( ( left == NULL ) || ( left->right != this ) )
		{
			cout << "left error " << hex << this << " " << this->left << endl;
			return TRUE;
		}
		if ( ( right == NULL ) || ( right->left != this ) )
		{
			cout << "right error " << hex << this << " " << this->right << endl;
			return TRUE;
		}
		return FALSE;
	}
	#if 0
	void LRInsertLeftOf ( HalfEdge* old ) 
	{
		right = old;
		left = old->left;
		old->left = this;
		inLR = TRUE;
	}
	#endif
	Real YStar () { return yStar; }
	Real XStar () { return v->X(); }
	Edge* GetEdge() { return e; }
	Vertex* GetVertex() { return v; }
	void SetVertex ( Vertex* newv, Real offset )
	{
		v = newv;
		v->Ref();
		yStar = v->Y() + offset;
	}
	Point VertexStar() { return Point ( v->X(), yStar ); }
	int Less ( HalfEdge* he )
	{
		return ( (yStar < he->yStar) ||
			((yStar == he->yStar) && (v->X() < he->v->X()) ) );
	}
	int RightOf ( Point& p );
	Vertex* Intersect ( HalfEdge* el2 );

	int Id() { return id; }
	int InLR() { return inLR; }
	void LeftEndSet ( HalfEdge* rightEnd )
	{
		inLR = TRUE;
		rightEnd->inLR = TRUE;
		right = rightEnd;
		rightEnd->left = this;
	}
	int PM() { return pm; }
	void Ref() { refCount++; }
	void Deref()
	{
		#if 0
		refCount--;
		if ( refCount == 0 )
		{
			if ( v )
				v->Deref();
			if ( e )
				e->Deref();
			delete this;
		}
		#endif
	}

private:
	Edge* e;
	Vertex* v;
	int pm; // LEFT_END or RIGHT_END
	Real yStar; // priority queue stuff?
	HalfEdge* left;
	HalfEdge* right;
	int inLR;

	int id;
	int refCount;
	int endPointCount;

	static int idCounter;
};

#endif
