static char SccsId[] = "@(#)voronoi.cc	1.1 12/12/96 Voronoi";
// voronoi.cc
// Dec 1996
// Copyright 1996 SFBellenot All rights reserved

#include "voronoi.h"
#include <X11/Xlib.h>
#include <iostream.h>
#include "pad.h"

Voronoi::Voronoi( Real XMin, Real XMax, Real YMin, Real YMax,
	SiteList* SList, int SqrtNumberOfSites )
{
	xMin = XMin;
	xMax = XMax;
	yMin = YMin;
	yMax = YMax;
	triangle = 0;

	Real dx = xMax - xMin;
	Real dy = yMax - yMin;
	Real d = max ( dx, dy ) * 1.1;
	pxMin = xMin - (d - dx) / 2.0;
	pxMax = xMax + (d - dx) / 2.0;
	pyMin = yMin - (d - dy) / 2.0;
	pyMax = yMax + (d - dy) / 2.0;
	cradius = ( pxMax - pxMin ) / 350.0;

	sl = SList;

	bottomSite = sl->NextSite();
	newSite = sl->NextSite();
	ySweep = newSite->Y();

	pq = new PQ ( SqrtNumberOfSites, YMin, YMax );
	lr = new LR ( SqrtNumberOfSites, XMin, XMax );

	complete = new HalfEdge ( NULL, 0 );
}

Voronoi::~Voronoi()
{
	if ( pq )
		delete pq;
	if ( lr )
		delete lr;
	// delete the complete list.
}

int Voronoi::Loop()
{
	Site* bot;
	Site* top;
	Site* temp;
	Vertex* p;
	Vertex* v;
	Point newIntStar;
	int pm;
	HalfEdge* lbnd;
	HalfEdge* rbnd;
	HalfEdge* llbnd;
	HalfEdge* rrbnd;
	HalfEdge* bisector;
	Edge *e;

	//lr->Print(ySweep);
	//pq->Print();

	if ( !pq->Empty() )
		newIntStar = pq->MinPoint();

	if ( newSite != NULL && ( pq->Empty() || newSite->LexLess ( newIntStar ) ) )
	{
		// new site is smallest
		ySweep = newSite->Y();
		//cout << "new site" << endl;
		//cout << "(" << newSite->X() << ", " << newSite->Y() << ")" << endl;
		//out_site(newSite);
		lbnd = lr->LeftBnd ( *newSite );
		rbnd = lbnd->RightHalfEdge();
		bot = lbnd->RightSite();
		if ( bot == NULL )
			bot = bottomSite;
		e = new Edge;
		e-> Bisect ( bot, newSite );
		bisector = new HalfEdge ( e, LEFT_END );
		bisector->LRInsertRightOf ( lbnd );
		p = lbnd->Intersect ( bisector );
		if ( p != NULL)
		{
			pq->Delete ( lbnd );
			pq->Insert ( lbnd, p, p->Distance( *newSite ) );
		}
		lbnd = bisector;
		bisector = new HalfEdge (e, RIGHT_END);
		bisector->LRInsertRightOf ( lbnd );
		p = rbnd->Intersect( bisector);
		if ( p != NULL )
		{
			pq->Insert( bisector, p, p->Distance ( *newSite ) );
		}
		newSite = sl->NextSite();
	}
	else if ( !pq->Empty() )
	{
		// intersection is smallest
		ySweep = newIntStar.Y();

		lbnd = pq->ExtractMin();
		llbnd = lbnd->LeftHalfEdge();
		rbnd = lbnd->RightHalfEdge();
		rrbnd = rbnd->RightHalfEdge();
		bot = lbnd->LeftSite();
		top = rbnd->RightSite();
		//out_triple(bot, top, lbnd->RightSite() );
		if ( triangle )
			cout << bot->Id() << " " << top->Id() << " "
				<< lbnd->RightSite()->Id() << endl;
		v = lbnd->GetVertex();
		//makevertex(v);
		lbnd->GetEdge()->EndPoint( lbnd->PM(), v );
		rbnd->GetEdge()->EndPoint( rbnd->PM(), v );
		lbnd->LRDelete();
		lbnd->InsertRightOfComplete ( complete );
		pq->Delete ( rbnd );
		rbnd->LRDelete();
		rbnd->InsertRightOfComplete ( complete );
		pm = LEFT_END;
		if (bot->Y() > top->Y())
		{
			temp = bot;
			bot = top;
			top = temp;
			pm = RIGHT_END;
		}
		e = new Edge;
		e->Bisect ( bot, top );
		bisector = new HalfEdge ( e, pm );
		bisector->LRInsertRightOf ( llbnd );
		e->EndPoint ( RIGHT_END - pm, v );
		v->Deref();
		p = llbnd->Intersect ( bisector );
		if ( p != NULL)
		{
			pq->Delete(llbnd);
			pq->Insert(llbnd, p, p->Distance ( *bot ) );
		}
		p = bisector->Intersect ( rrbnd );
		if ( p != NULL)
		{
			pq->Insert(bisector, p, p->Distance ( *bot ) );
		}
	}
	else
	{
		ySweep = pyMax;
		return 0;
	}
	return 1;
}

void Voronoi::Circle ( Point& p )
{
	PadCircle ( pad, (p.X() - pxMin) * scale, (p.Y() - pyMin) * scale,
		cradius * scale );
}

void Voronoi::Line ( Real x1, Real y1, Real x2, Real y2 )
{
	PadLine ( pad, (x1 - pxMin) * scale, (y1 - pyMin) * scale,
		(x2 - pxMin) * scale, (y2 - pyMin) * scale );
}

void Voronoi::Draw ( Pad* p )
{
	pad = p;
	scale = min ( PadWidth ( p ) / (pxMax - pxMin ),
		PadHeight ( p ) / ( pyMax - pyMin ) );

	PadClear ( p );
	if ( ySweep < pyMax )
	{
		PadFgColor ( p, PadGetColor ( p, "blue" ) );
		Line ( pxMin, ySweep, pxMax, ySweep );
		PadFgColor ( p, PadGetColor ( p, "red" ) );
	}
	//foreach edge draw edge
	//foreach site draw site
	HalfEdge* he = lr->LeftMost();
	Edge* e = NULL;
	if ( he )
		e = he->GetEdge();
	for ( ; he && e ; )
	{
		e->Draw ( this );
		if ( e->LeftSite() )
			Circle ( *e->LeftSite() );
		if ( e->RightSite() )
			Circle ( *e->RightSite() );
		he = he->RightHalfEdge();
		if ( he )
			e = he->GetEdge();
	}
	he = complete->RightHalfEdge();
	if ( he )
		e = he->GetEdge();
	else
		e = NULL;
	for ( ; he && e ; )
	{
		e->Draw ( this );
		if ( e->LeftSite() )
			Circle ( *e->LeftSite() );
		if ( e->RightSite() )
			Circle ( *e->RightSite() );
		he = he->RightHalfEdge();
		if ( he )
			e = he->GetEdge();
	}
}

void Voronoi::End()
{
	
}
